# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import steps.model as smod
import steps.geom as sgeom
import steps.rng as srng
import steps.solver as ssolv

import math
import time 
import numpy
from pylab import *

from tol_funcs import *

########################################################################

VOL = 1.0e-18			#4.1533524410151228e-18

COUNTA = 100.0
n=2.0
COUNTB = COUNTA/n 


KCST = 10.0e10			# The reaction constant

AREA = 10.0e-12

CCST = KCST/(6.02214179e23*AREA)


NITER = 1000			# The number of iterations
DT = 0.05			# Sampling time-step
INT = 1.05			# Sim endtime

# In tests fewer than 0.1% fail with tolerance of 2%
tolerance = 2.0/100

########################################################################

mdl  = smod.Model()

A = smod.Spec('A', mdl)
B = smod.Spec('B', mdl)
C = smod.Spec('C', mdl)

surfsys = smod.Surfsys('ssys',mdl)

SR1 = smod.SReac('SR1', surfsys, slhs = [A, B], srhs = [C], kcst = KCST)

geom = sgeom.Geom()

comp1 = sgeom.Comp('comp1', geom, VOL)
patch1 = sgeom.Patch('patch1', geom, comp1, area = AREA)
patch1.addSurfsys('ssys')

import random
rng = srng.create('mt19937', 1000)
rng.initialize(int(random.random()*4294967295))


sim = ssolv.Wmdirect(mdl, geom, rng)
sim.reset()

tpnts = numpy.arange(0.0, INT, DT)
ntpnts = tpnts.shape[0]

res_m = numpy.zeros([NITER, ntpnts, 3])

for i in xrange (0, NITER):
	sim.reset()
	sim.setPatchCount('patch1', 'A', COUNTA)
	sim.setPatchCount('patch1', 'B', COUNTB)

	for t in xrange(0, ntpnts):
		sim.run(tpnts[t])
		res_m[i, t, 0] = sim.getPatchCount('patch1', 'A')
		res_m[i, t, 1] = sim.getPatchCount('patch1', 'B')       

mean_res = numpy.mean(res_m, 0)


lnBA = numpy.zeros(ntpnts)
lineAB = numpy.zeros(ntpnts)

C = COUNTA-COUNTB
passed  =True
max_err = 0.0

for i in range(ntpnts):
    A = mean_res[i][0]
    B = mean_res[i][1]
    lnBA[i] = math.log(B/A)
    lineAB[i] = math.log(COUNTB/COUNTA) -C*CCST*tpnts[i]
    if not tolerable(lnBA[i], lineAB[i], tolerance): passed = False
    #if (abs(2*(lnBA[i]-lineAB[i])/(lnBA[i]+lineAB[i])) > max_err): max_err = abs(2*(lnBA[i]-lineAB[i])/(lnBA[i]+lineAB[i]))
#print "max error", max_err*100.0, "%"




# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

#  Last Changed Rev:  $Rev: 352 $
#  Last Changed Date: $Date: 2010-08-09 10:49:54 +0900 (Mon, 09 Aug 2010) $
#  Last Changed By:   $Author: wchen $

try:
    import cubit
except ImportError:
    print("Unable to find CUBIT module.")
    
import random
import os
import sys

################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################

def getTetsInVolume(v_id, mesh, scale = 1e-6):
    ntets = mesh.ntets
    volume = cubit.volume(v_id)
    body = volume.bodies()[0]
    in_list = []
    for t in range(ntets):
        center = mesh.getTetBarycenter(t)
        cubit_center = [center[0]/scale, center[1]/scale, center[2]/scale]
        status = body.point_containment(cubit_center)
        if status == 1 or status == 2:
            in_list.append(t)
    return in_list
 
################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################

def getCompTetsInVolume(v_id, mesh, comp_id, scale = 1e-6):
    comp_tets = mesh.getComp(comp_id).getAllTetIndices()
    volume = cubit.volume(v_id)
    body = volume.bodies()[0]
    in_list = []
    for t in comp_tets:
        center = mesh.getTetBarycenter(t)
        cubit_center = [center[0]/scale, center[1]/scale, center[2]/scale]
        status = body.point_containment(cubit_center)
        if status == 1 or status == 2:
            in_list.append(t)
    return in_list

################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################
        
def getSurfTrisInVolume(v_id, mesh, scale = 1e-6):
    surf_tris = mesh.getSurfTris()
    volume = cubit.volume(v_id)
    body = volume.bodies()[0]
    in_list = []
    for t in surf_tris:
        center = mesh.getTriBarycenter(t)
        cubit_center = [center[0]/scale, center[1]/scale, center[2]/scale]
        status = body.point_containment(cubit_center)
        if status == 1 or status == 2:
            in_list.append(t)
    return in_list
 
################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################

def getPatchTrisInVolume(v_id, mesh, patch_id, scale = 1e-6):
    patch_tris = mesh.getPatch(patch_id).getAllTriIndices()
    volume = cubit.volume(v_id)
    body = volume.bodies()[0]
    in_list = []
    for t in patch_tris:
        center = mesh.getTriBarycenter(t)
        cubit_center = [center[0]/scale, center[1]/scale, center[2]/scale]
        status = body.point_containment(cubit_center)
        if status == 1 or status == 2:
            in_list.append(t)
    return in_list

################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################
    
def highlightTets(steps_tets, tet_proxy):
    cubit.cmd("Graphics Clear Highlight")
    cubit.cmd("Graphics Pause")
    cmd_str = "highlight tet "
    for st in steps_tets:
        cubit_id = tet_proxy.getImportID(st)
        cmd_str += "%i," % (cubit_id)
    cubit.cmd(cmd_str)
    cubit.cmd("Display")

################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################
    
class VSim(object):
    def __init__(self, model, mesh, sim, scale = 1e-6,color_map = None,
                cubit_file = None, visual_mode = "wireframe", with_mesh = False):
        if cubit_file != None:
            cubit.cmd('open "%s"' % (cubit_file))
        
        cubit.cmd(visual_mode)
        if with_mesh:
            cubit.cmd("mesh on")
        else:
            cubit.cmd("mesh off")
        self.model = model
        self.mesh = mesh
        self.sim = sim
        self.scale = scale

        self.specs = []
        spec_refs = model.getAllSpecs()
        for spec in spec_refs:
            self.specs.append(spec.getID())

        colors = random.sample(range(86), len(self.specs))

        self.colors = {}
        for i in range(len(colors)):
            self.colors[self.specs[i]] = colors[i]
            
        if color_map != None:
            for s in color_map:
                self.colors[s] = color_map[s]
                
        comp_refs = mesh.getAllComps()
        self.comp_tets = {}
        for c in comp_refs:
            self.comp_tets[c.getID()] = []
        for t in range(mesh.ntets):
            comp = self.mesh.getTetComp(t)
            if comp != None:
                self.comp_tets[comp.getID()].append(t)
                
        patch_refs = mesh.getAllPatches()
        self.patch_tris = {}
        for p in patch_refs:
            self.patch_tris[p.getID()] = []
        for t in range(mesh.ntris):
            patch = self.mesh.getTriPatch(t)
            if patch != None:
                self.patch_tris[patch.getID()].append(t)
                
    def changeColor(self, spec_id, color_idx):
        self.colors[spec] = color_idx
    
    def run(self, end_time, update_interval, record_file = None):
        if record_file != None:
            rc_file = open(record_file, 'w')
            rc_file.seek(os.SEEK_END)
        while self.sim.getTime() < end_time:
            cubit.cmd("Graphics Pause")
            cubit.cmd("delete vertex all")
            if record_file != None:
                rc_file.write("Graphics Pause\n")
                rc_file.write("delete vertex all\n")
            self.sim.advance(update_interval)
            for comp in self.comp_tets.values():
                for t in comp:
                    for spec in self.specs:
                        if(self.sim.getTetCount(t, spec) > 0):
                            center = self.mesh.getTetBarycenter(t)
                            scaled_center = [center[0]/self.scale, 
                                            center[1]/self.scale, 
                                            center[2]/self.scale]
                            cmd = "create vertex %f %f %f color id %i" % (scaled_center[0],
                            scaled_center[1], scaled_center[2], self.colors[spec])
                            cubit.cmd(cmd)
                            if record_file != None:
                                rc_file.write(cmd + "\n")

            for patch in self.patch_tris.values():
                for t in patch:
                    for spec in self.specs:
                        if(self.sim.getTriCount(t, spec) > 0):
                            center = self.mesh.getTriBarycenter(t)
                            scaled_center = [center[0]/self.scale, 
                                            center[1]/self.scale, 
                                            center[2]/self.scale]
                            cmd = "create vertex %f %f %f color id %i" % (scaled_center[0],
                            scaled_center[1], scaled_center[2], self.colors[spec])
                            cubit.cmd(cmd)
                            if record_file != None:
                                rc_file.write(cmd + "\n")
            cubit.cmd("display")
            rc_file.write("display\n")
        if record_file != None:
            rc_file.close()
################################################################################
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
################################################################################            


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import steps.model as smod
import steps.geom as sgeom
import steps.rng as srng
import steps.solver as ssolv

import numpy
from pylab import *
import time

from tol_funcs import *

########################################################################

KCST = 5			# The reaction constant
N = 50				# Can set count or conc
VOL = 1.0e-18

NITER = 100000		# The number of iterations
DT = 0.1			# Sampling time-step
INT = 1.1			# Sim endtime

# Tolerance for the comparison:
# In test runs, with good code, < 1%  will fail with a 1.5% tolerance
tolerance = 2.0/100  

########################################################################

mdl  = smod.Model()

A = smod.Spec('A', mdl)
volsys = smod.Volsys('vsys',mdl)
R1 = smod.Reac('R1', volsys, lhs = [A], rhs = [], kcst = KCST)


geom = sgeom.Geom()
comp1 = sgeom.Comp('comp1', geom, VOL)
comp1.addVolsys('vsys')

rng = srng.create('mt19937', 1000)
rng.initialize(int(time.time()%4294967295))


sim = ssolv.Wmdirect(mdl, geom, rng)
sim.reset()

tpnts = numpy.arange(0.0, INT, DT)
ntpnts = tpnts.shape[0]

res_m = numpy.zeros([NITER, ntpnts, 1])
res_std1 = numpy.zeros([ntpnts, 1])
res_std2 = numpy.zeros([ntpnts, 1])

for i in range (0, NITER):
	sim.reset()
	sim.setCompCount('comp1', 'A', N)

	for t in range(0, ntpnts):
		sim.run(tpnts[t])
		res_m[i, t, 0] = sim.getCompCount('comp1', 'A')

mean_res = numpy.mean(res_m, 0)
std_res = numpy.std(res_m, 0)

m_tol = 0
s_tol=0

passed = True
for i in range(ntpnts):
    if i == 0: continue
    analy = N*math.exp(-KCST*tpnts[i])
    std = math.pow((N*(math.exp(-KCST*tpnts[i]))*(1-(math.exp(-KCST*tpnts[i])))), 0.5)
    if not tolerable(analy, mean_res[i], tolerance):
        passed = False
    if not tolerable(std, std_res[i], tolerance): 
        passed = False





# -*- coding: utf-8 -*-

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

#  Last Changed Rev:  $Rev: 410 $
#  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
#  Last Changed By:   $Author: iain $

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# This file is the user-interface file for all model objects in steps.
# All objects are directly derived from the corresponding swig objects.
# Model container object is owned by Python
# All other objects are owned by c++ and Model container is responsible for 
# all the cleaning-up of these objects (see cpp/model/model.cpp class 
# destructor).
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


from . import steps_swig
import _steps_swig



class Model(steps_swig.Model) : 
    """
    Top-level container for the objects in a kinetic model.
    
    """
    
    def __init__(self, *args):
        """
        Construction::
        
            m = steps.model.Model()
            
        Create a model container object.
            
        Arguments: 
            None
        """
        
        this = _steps_swig.new_Model(*args)
        try: self.this.append(this)
        except: self.this = this
        # let the model object do the cleaning-up
        self.thisown = True

class Spec(steps_swig.Spec) :

    """
    A chemical species which can be a reactant and/or product in reaction 
    stoichiometry and/or associated with a diffusion rule. 
    """
    def __init__(self, *args): 
        """        
        Construction::
        
            s = steps.model.Spec(id, mdl)
            
        Create a species object with identifier string id and assign the 
        object mdl as its parent model.
            
        Arguments: 
            * string id
            * steps.model.Model mdl
        """
        this = _steps_swig.new_Spec(*args)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Spec_setID
        self.__swig_getmethods__["id"] = _steps_swig.Spec_getID
        self.__swig_getmethods__["model"] = _steps_swig.Spec_getModel
    id = steps_swig._swig_property(_steps_swig.Spec_getID, _steps_swig.Spec_setID)
    """Identifier string of the species."""
    model = steps_swig._swig_property(_steps_swig.Spec_getModel)
    """Reference to parent model."""

class Surfsys(steps_swig.Surfsys):
    """
    A container that groups reactions involving a reactant embedded in a membrane.
    """
    def __init__(self, *args): 
        """
        Construction::
        
            s = steps.model.Surfsys(id, mdl)
            
        Construct a surface system object with identifier string id and assign 
        the object mdl as its parent model.
        
        Arguments: 
            * string id
            * steps.model.Model mdl
        """
        this = _steps_swig.new_Surfsys(*args)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Surfsys_setID
        self.__swig_getmethods__["id"] = _steps_swig.Surfsys_getID
        self.__swig_getmethods__["model"] = _steps_swig.Surfsys_getModel
    id = steps_swig._swig_property(_steps_swig.Surfsys_getID, _steps_swig.Surfsys_setID)
    """Identifier string of the surface system."""
    model = steps_swig._swig_property(_steps_swig.Surfsys_getModel)
    """Reference to parent model."""

class Volsys(steps_swig.Volsys) :
    """
    A container that groups reactions involving reactants embedded in a volume.
    """
    def __init__(self, *args): 
        """
        Construction::
        
            v = steps.model.Volsys(id, mdl)
            
        Construct a volume system object with identifier string id and assign 
        the object mdl as its parent model.
        
        Arguments: 
            * string id
            * steps.model.Model mdl
        """
        this = _steps_swig.new_Volsys(*args)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Volsys_setID
        self.__swig_getmethods__["id"] = _steps_swig.Volsys_getID
        self.__swig_getmethods__["model"] = _steps_swig.Volsys_getModel
    id = steps_swig._swig_property(_steps_swig.Volsys_getID, _steps_swig.Volsys_setID)
    """Identifier string of the volume system."""
    model = steps_swig._swig_property(_steps_swig.Volsys_getModel)
    """Reference to parent model."""

class Diff(steps_swig.Diff) :
    """
    A diffusion rule for a chemical species in a volume.
    
    A diffusion rule is described by:
    * Species to which the diffusion rule applies (lig).
    * Diffusion constant (dcst) specified in s.i. units.
    """
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            diff = steps.model.Diff(id, volsys, lig, dcst = 0.0)
            
        Construct a diffusion rule object with identifier string id applied to species 
        lig and assign volsys as the parent volume system. Diffusion constant is 
        set by dcst.
        
        Arguments: 
            * string id
            * steps.model.Volsys volsys
            * steps.model.Spec lig
            * float dcst (default = 0.0)
        """
        this = _steps_swig.new_Diff(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Diff_setID
        self.__swig_getmethods__["id"] = _steps_swig.Diff_getID
        self.__swig_getmethods__["model"] = _steps_swig.Diff_getModel
        self.__swig_getmethods__["volsys"] = _steps_swig.Diff_getVolsys
        self.__swig_setmethods__["lig"] = _steps_swig.Diff_setLig
        self.__swig_getmethods__["lig"] = _steps_swig.Diff_getLig
        self.__swig_setmethods__["dcst"] = _steps_swig.Diff_setDcst
        self.__swig_getmethods__["dcst"] = _steps_swig.Diff_getDcst
    id = steps_swig._swig_property(_steps_swig.Diff_getID, _steps_swig.Diff_setID)
    """Identifier string of the diffusion."""
    model = steps_swig._swig_property(_steps_swig.Diff_getModel)
    """Reference to parent model."""
    volsys = steps_swig._swig_property(_steps_swig.Diff_getVolsys)
    """Reference to parent volume system."""
    dcst = steps_swig._swig_property(_steps_swig.Diff_getDcst, _steps_swig.Diff_setDcst)
    """Diffusion constant."""
    lig = steps_swig._swig_property(_steps_swig.Diff_getLig, _steps_swig.Diff_setLig)
    """Reference to diffusion species."""

class Reac(steps_swig.Reac) :
    """
    A kinetic reaction rule in a volume.
    
    The reaction rule is specified by:
    * Species on the left hand side of the reaction: the reactants (lhs).
    * Species on the right hand side of the reaction: the products (rhs).
    * Rate constant for the reaction, supplied in s.i. units (kcst).
    """
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            reac = steps.model.Reac(id, volsys, lhs = [ ], rhs = [ ], kcst = 0.0)
            
        Construct a reaction rule object with identifier string id and assign 
        volsys as the parent volume system. A list of left hand side reactants 
        may be assigned with lhs, whilst a list of right hand side products may 
        be assigned with rhs, the kinetic reaction rate constant is set by kcst.
        
        Arguments: 
            * string id
            * steps.model.Volsys volsys
            * list(steps.model.Spec) lhs (default = [ ]) 
            * list(steps.model.Spec) rhs (default = [ ]) 
            * float kcst (default = 0.0)
        """
        this = _steps_swig.new_Reac(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Reac_setID
        self.__swig_getmethods__["id"] = _steps_swig.Reac_getID
        self.__swig_getmethods__["model"] = _steps_swig.Reac_getModel
        self.__swig_getmethods__["volsys"] = _steps_swig.Reac_getVolsys
        self.__swig_setmethods__["kcst"] = _steps_swig.Reac_setKcst
        self.__swig_getmethods__["kcst"] = _steps_swig.Reac_getKcst
        self.__swig_setmethods__["lhs"] = _steps_swig.Reac_setLHS
        self.__swig_getmethods__["lhs"] = _steps_swig.Reac_getLHS
        self.__swig_setmethods__["rhs"] = _steps_swig.Reac_setRHS
        self.__swig_getmethods__["rhs"] = _steps_swig.Reac_getRHS
        self.__swig_getmethods__["order"] = _steps_swig.Reac_getOrder
    id = steps_swig._swig_property(_steps_swig.Reac_getID, _steps_swig.Reac_setID)
    """Identifier string of the reaction."""
    model = steps_swig._swig_property(_steps_swig.Reac_getModel)
    """Reference to parent model."""
    volsys = steps_swig._swig_property(_steps_swig.Reac_getVolsys)
    """Reference to parent volume system."""
    kcst = steps_swig._swig_property(_steps_swig.Reac_getKcst, _steps_swig.Reac_setKcst)
    """Reaction constant."""
    lhs = steps_swig._swig_property(_steps_swig.Reac_getLHS, _steps_swig.Reac_setLHS)
    """Left hand side reactants."""
    rhs = steps_swig._swig_property(_steps_swig.Reac_getRHS, _steps_swig.Reac_setRHS)
    """Right hand side reactants."""
    order = steps_swig._swig_property(_steps_swig.Reac_getOrder)
    """Order of the reaction."""
    
class SReac(steps_swig.SReac) :
    """
    A reaction rule where at least one reactant is embedded in a surface. 

    In a surface reaction, the species can be classified as:
    
    * Reactants on the left hand side of the reaction:
        * Species in the 'outer' compartment (olhs)
        * or Species in the 'inner' compartment (ilhs)
        * Species on the surface (slhs)
    * Reactants on right hand side of the reaction:
        * Species in the 'outer' compartment (orhs)
        * Species in the 'inner' compartment (irhs)
        * Species on the surface (srhs)
    * The reaction rate is defined by kcst, supplied in s.i. units.
    """
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            sreac = steps.model.Reac(id, surfsys, 
                                    ilhs = [ ], olhs = [ ], slhs = [ ],
                                    irhs = [ ], orhs = [ ], srhs = [ ],
                                    kcst = 0.0)
            
        Construct a surface reaction rule object with identifier string 
        id and assign surfsys as the parent surface system. A list of 
        left hand reactants are assigned with ilhs, olhs and slhs 
        (default for each is an empty list). A list of right hand side 
        products are assigned with irhs, orhs and srhs (default for each 
        is an empty list). The kinetic reaction rate constant is set with kcst.
        
        
        Arguments: 
            * string id
            * steps.model.Surfsys surfsys
            * list(steps.model.Spec) ilhs (default = [ ]) 
            * list(steps.model.Spec) olhs (default = [ ])
            * list(steps.model.Spec) slhs (default = [ ])  
            * list(steps.model.Spec) irhs (default = [ ])
            * list(steps.model.Spec) orhs (default = [ ])
            * list(steps.model.Spec) srhs (default = [ ])
            * float kcst (default = 0.0)
        """
        this = _steps_swig.new_SReac(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.SReac_setID
        self.__swig_getmethods__["id"] = _steps_swig.SReac_getID
        self.__swig_getmethods__["model"] = _steps_swig.SReac_getModel
        self.__swig_getmethods__["surfsys"] = _steps_swig.SReac_getSurfsys
        self.__swig_setmethods__["kcst"] = _steps_swig.SReac_setKcst
        self.__swig_getmethods__["kcst"] = _steps_swig.SReac_getKcst
        self.__swig_getmethods__["order"] = _steps_swig.SReac_getOrder
        self.__swig_getmethods__["outer"] = _steps_swig.SReac_getOuter
        self.__swig_setmethods__["olhs"] = _steps_swig.SReac_setOLHS
        self.__swig_getmethods__["olhs"] = _steps_swig.SReac_getOLHS
        self.__swig_setmethods__["ilhs"] = _steps_swig.SReac_setILHS
        self.__swig_getmethods__["ilhs"] = _steps_swig.SReac_getILHS
        self.__swig_setmethods__["slhs"] = _steps_swig.SReac_setSLHS
        self.__swig_getmethods__["slhs"] = _steps_swig.SReac_getSLHS
        self.__swig_setmethods__["irhs"] = _steps_swig.SReac_setIRHS
        self.__swig_getmethods__["irhs"] = _steps_swig.SReac_getIRHS
        self.__swig_setmethods__["srhs"] = _steps_swig.SReac_setSRHS
        self.__swig_getmethods__["srhs"] = _steps_swig.SReac_getSRHS
        self.__swig_setmethods__["orhs"] = _steps_swig.SReac_setORHS
        self.__swig_getmethods__["orhs"] = _steps_swig.SReac_getORHS
    id = steps_swig._swig_property(_steps_swig.SReac_getID, _steps_swig.SReac_setID)
    """Identifier string of the surface reaction."""
    model = steps_swig._swig_property(_steps_swig.SReac_getModel)
    """Reference to parent model."""
    surfsys = steps_swig._swig_property(_steps_swig.SReac_getSurfsys)
    """Reference to parent surface system."""
    kcst = steps_swig._swig_property(_steps_swig.SReac_getKcst, _steps_swig.SReac_setKcst)
    """Reaction constant."""
    outer = steps_swig._swig_property(_steps_swig.SReac_getOuter)
    """Obsolete"""
    olhs = steps_swig._swig_property(_steps_swig.SReac_getOLHS, _steps_swig.SReac_setOLHS)
    """Left hand side reactants in outer compartment."""
    ilhs = steps_swig._swig_property(_steps_swig.SReac_getILHS, _steps_swig.SReac_setILHS)
    """Left hand side reactants in inner compartment."""
    slhs = steps_swig._swig_property(_steps_swig.SReac_getSLHS, _steps_swig.SReac_setSLHS)
    """Left hand side reactants on surface."""
    irhs = steps_swig._swig_property(_steps_swig.SReac_getIRHS, _steps_swig.SReac_setIRHS)
    """Right hand side reactants in inner compartment."""
    srhs = steps_swig._swig_property(_steps_swig.SReac_getSRHS, _steps_swig.SReac_setSRHS)
    """Right hand side reactants on surface."""
    orhs = steps_swig._swig_property(_steps_swig.SReac_getORHS, _steps_swig.SReac_setORHS)
    """Right hand side reactants in outer compartment."""
    order = steps_swig._swig_property(_steps_swig.SReac_getOrder)
    """Order of the reaction."""


